#!/usr/bin/perl
use strict;
use warnings;
use Math::MPFI qw(:mpfi);

Rmpfi_set_default_prec(32);

sub one_step_newton{
  my ($fref, $fpref, $x0) = @_;
 
  my $m = Math::MPFR->new();
  Rmpfi_mid($m, $x0);                  # $m midpoint of x0
  my $mi = Math::MPFI->new($m);        # idem but MPFI object
  
  my $fm = $fref->($mi);               # $fm = f($mi)
  my $fp = $fpref->($x0);              # $fp = f'($x0)
  
  if (Rmpfi_has_zero($fp)) {
    print "f\'(x0) has zero\n";
  }
  
  my $x1 = $mi-$fm/$fp;
  Rmpfi_intersect($x1, $x1, $x0);
  if (Rmpfi_is_empty($x1)) {
    die "No solution in $x0\n";
  }  
  return $x1;
}

sub f{
  return $_[0]*$_[0]-3;
}

sub fp{
  return 2*$_[0];
}

my $x0 = Math::MPFI->new('[1,2]');
my $x1 = $x0;

do {
  $x0 = $x1;
  print "$x1\n";
  $x1 = one_step_newton(\&f,\&fp, $x0); 
} until (Rmpfi_is_inside($x0, $x1));
