#!/usr/bin/perl
use strict;
use warnings;
use Math::MPFI qw(:mpfi);

sub xreciprocal {
  my $x = shift;                                 # $x the argument
  my @r;                                         # @r the result (a list)
  if (Rmpfi_is_zero($x)) {
    push @r, Math::MPFI->new('[@NaN@,@NaN@]');
  }
  elsif (Rmpfi_has_zero($x)) {
    my $zero = Math::MPFR->new(0);
    my $inf = Math::MPFR->new();
    Rmpfi_get_left($inf, $x);
    if ($inf != 0) {
      my $xinf = Math::MPFI->new();
      Rmpfi_interv_fr($xinf, $inf, $zero); 
      push @r, 1/$xinf;
    }
    my $sup = Math::MPFR->new();
    Rmpfi_get_right($sup, $x); 
    if ($sup != 0) {
      my $xsup = Math::MPFI->new();
      Rmpfi_interv_fr($xsup, $zero, $sup); 
      push @r, 1/$xsup;
    }
  }
  else {
    push @r, 1/$x;
  }
  return @r;
}

for ('[1,2]', '[-2,-1]', '[0,2]', '[-2,0]', '[-2,2]', 0) {
  my $x = Math::MPFI->new($_);
  print "1/$x \t= ", xreciprocal($x), "\n";
}

for ('[-@Inf@, -1]', '[-@Inf@, 0]', '[-@Inf@, 1]') {
  my $x = Math::MPFI->new($_);
  print "1/$x \t= ", xreciprocal($x), "\n";
}

for ('[-1, @Inf@]', '[0, @Inf@]', '[1, @Inf@]') {
  my $x = Math::MPFI->new($_);
  print "1/$x \t= ", xreciprocal($x), "\n";
}
