#!/usr/bin/perl
use strict;
use warnings;
use Math::MPFI qw(:mpfi);

Rmpfi_set_default_prec(64);

sub xreciprocal {
  my $x = shift;                                 # $x the argument
  my @r;                                         # @r the result (a list)
  if (Rmpfi_is_zero($x)) {
    push @r, Math::MPFI->new('[@NaN@,@NaN@]');
  }
  elsif (Rmpfi_has_zero($x)) {
    my $zero = Math::MPFR->new(0);
    my $inf = Math::MPFR->new();
    Rmpfi_get_left($inf, $x);
    if ($inf != 0) {
      my $xinf = Math::MPFI->new();
      Rmpfi_interv_fr($xinf, $inf, $zero); 
      push @r, 1/$xinf;
    }
    my $sup = Math::MPFR->new();
    Rmpfi_get_right($sup, $x); 
    if ($sup != 0) {
      my $xsup = Math::MPFI->new();
      Rmpfi_interv_fr($xsup, $zero, $sup); 
      push @r, 1/$xsup;
    }
  }
  else {
    push @r, 1/$x;
  }
  return @r;
}
sub solve_newton{
  my ($fref, $fpref, $x0) = @_;
  my @r;
  my $m = Math::MPFR->new();
  Rmpfi_mid($m, $x0);                           
  $m = Math::MPFI->new($m);
  my $fm = $fref->($m);
  my $fp = $fpref->($x0);
  if (Rmpfi_is_zero($fp)) {
    if (Rmpfi_is_zero($fm)) {
      return ($x0);
    }
    else {
      return ();
    }
  }
  foreach( xreciprocal($fp) ) {
    my $x1 = $m-$fm*$_;
    Rmpfi_intersect($x1, $x1, $x0);
    if (Rmpfi_is_empty($x1)) {
      next;
    }
    if (Rmpfi_is_inside($x0, $x1)) {
      if (Rmpfi_has_zero($fp)) {
        print "Sorry. Unable to isolate solutions. Accuracy too low?";
        exit;
      }
      push @r, $x1;
    }
    else {
      push @r, solve_newton($fref, $fpref, $x1);
    }
  }
  return @r
}

sub f{
  my $x2 = $_[0]*$_[0];
  return $x2+sin(1/$x2/$_[0]);
}

sub fp{
  my $x2 = $_[0]*$_[0];
  my $x4 = $x2*$x2;
  return 2*$_[0]-3*cos(1/$x2/$_[0])/$x4;
}

my $x0 = Math::MPFI->new('[0.1,1]');

my $i = '001';
print map {$i++." $_\n"}
      sort {$a<=>$b}
      solve_newton(\&f,\&fp, $x0); 
